/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import "ObjectCell.h"
#import "IDLDefinition_Private.h"
#import "IDLViewer.h"

@implementation NSDictionary(LoadCell)

- (void) fillMatrix:matrix
{
    NSArray* strings = [[self allKeys] sortedArrayUsingSelector:@selector(compare:)];
    NSEnumerator* en = [strings objectEnumerator];
    NSString* type;
    int n = [matrix numberOfRows];

    while(type = [en nextObject]){
        NSObject* obj = [self objectForKey:type];
        [matrix addRow];
        [[matrix cellAtRow:n column:0]setStringValue:type];
        [obj loadCell:[matrix cellAtRow:n column:0]];
        n += 1;
    }	
}

- (void) loadCell:cell
{
    [cell setLoaded:YES];
    [cell setLeaf:NO];
    [cell setRepresentedObject:self];
}

- (NSString*) title
{
    return @"List";
}

- (NSString*) stringValue
{
    return @"List";
}

@end

@implementation NSArray(LoadCell)

- (void) fillMatrix:matrix
{
    NSEnumerator* en = [self objectEnumerator];
    id object;
    int n = [matrix numberOfRows];
    while(object = [en nextObject]){
        id cell;
        [matrix addRow];
        cell = [matrix cellAtRow:n++ column:0];
        if([object respondsToSelector:@selector(name)])
            [cell setStringValue:[object name]];
        else
            [cell setStringValue:[object description]];
        if([object respondsToSelector:@selector(loadCell:)])
            [object loadCell:cell];
        else{
            [cell setLeaf:YES];
            [cell setRepresentedObject:object];
            [cell setLoaded:YES];
        }
    }
}

- (void) loadCell:cell
{
    [cell setLoaded:YES];
    [cell setLeaf:NO];
    [cell setRepresentedObject:self];
	[cell setImage:[NSImage imageNamed:@"array13"]];
}

- (NSString*) title
{
    return @"List";
}

- (NSString*) stringValue
{
    return @"List";
}

@end

@interface NSObject(RecursiveDesc)
- (NSString*) _recursiveDesc:(NSMutableArray*)array;
- (NSString*) recursiveDescription;
@end

@implementation NSObject(RecursiveDesc)

- (NSString*) _recursiveDesc:(NSMutableArray*)array
{
    return [self description];
}

- (NSString*) recursiveDescription
{
    return [self _recursiveDesc:[NSMutableArray array]];
}

@end

@implementation NSDictionary(RecursiveDesc)

- (NSString*) _recursiveDesc:(NSMutableArray*)array
{
    NSEnumerator* en = [self keyEnumerator];
    NSString* name;
    NSMutableString* str = [NSMutableString stringWithFormat:@"0x%x {\n", self];
    
    while(name = [en nextObject]){
        NSObject* obj = [self objectForKey:name];
        if([array containsObject:obj]){
            [str appendFormat:@"%@ = 0x%x;\n", name, obj];
        }else{
            if([obj isKindOfClass:[NSDictionary class]])
                [array addObject:obj];
            [str appendFormat:@"%@ = %@;\n", name, [obj _recursiveDesc:array]];
        }
    }
    [str appendString:@"}"];
    return str;
}

@end
