/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import <AppKit/AppKit.h>
#import "IDLPreferences.h"
#import "IDLDefinition_Private.h"

@implementation IDLPreferences


+ (void) initialize
{
    if([IDLPreferences class] == self){
        NSBundle* bundle = [NSBundle bundleForClass:self];
        NSString* path = [bundle pathForResource:@"IDLPreferences" ofType:@"plist"];
        if(path){
            NSDictionary* dict = [[NSDictionary alloc]initWithContentsOfFile:path];
            if(dict)
                [[NSUserDefaults standardUserDefaults]registerDefaults:dict];
            [dict release];
        }
    }
}

- (void) setApplication:sender
{
    NSWorkspace* ws = [NSWorkspace sharedWorkspace];
    NSOpenPanel* openPanel = [NSOpenPanel openPanel];
    NSString* app = [[NSUserDefaults standardUserDefaults]stringForKey:@"IDLEditApp"];
    NSString* appPath = app;
    NSString* directory;
    NSString* file;
    if([app characterAtIndex:0] != '/')
        appPath = [ws fullPathForApplication:app];
    directory = [appPath stringByDeletingLastPathComponent];
    file = [appPath lastPathComponent];
    [openPanel setTreatsFilePackagesAsDirectories:NO];
    [openPanel setAllowsMultipleSelection:NO];
	[openPanel beginSheetForDirectory:directory file:file types:[NSArray arrayWithObject:@"app"]
			modalForWindow:window modalDelegate:self
			didEndSelector:@selector(openSheetDidEnd:returnCode:contextInfo:)
			contextInfo:nil];
}

- (void) openSheetDidEnd:(NSOpenPanel*)openPanel returnCode:(int)rc contextInfo:(void*)info
{
	if(rc){
		NSWorkspace* ws = [NSWorkspace sharedWorkspace];
        NSArray* files = [openPanel filenames];
        NSString* appPath = [files objectAtIndex:0];
        [[NSUserDefaults standardUserDefaults]setObject:appPath forKey:@"IDLEditApp"];
        NSImage* image = [ws iconForFile:appPath];
        [appIconView setImage:image];
	}
}

- (void) setBrowserColumnCount:sender
{
    int n = [columnCount intValue];
    if(n >=2 && n < 7){
        [[NSUserDefaults standardUserDefaults]setInteger:n forKey:@"ColumnCount"];
    }else{
        NSBeep();
        [columnCount setIntValue:[[NSUserDefaults standardUserDefaults]integerForKey:@"ColumnCount"]];
    }
}

- (void) orderFront:sender
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    int n = [defaults integerForKey:@"ColumnCount"];
    int cnt = [IDLDefinition sortIfGreaterThan];
    BOOL shouldSort = [IDLDefinition sortMatrixItems];
    NSWorkspace* ws = [NSWorkspace sharedWorkspace];
    NSString* app = [defaults stringForKey:@"IDLEditApp"];
    NSString* path = [ws fullPathForApplication:app];
    NSImage* image = nil;
    if(path)
        image = [ws iconForFile:path];
    [appIconView setImage:image];
    [columnCount setIntValue:n];
    [columnCount setEntryType:NSIntType];
    [countField setIntValue:cnt];
    [[countField selectedCell]setEntryType:NSIntType];
    if(shouldSort){
        [sortMatrix selectCellWithTag:1];
        [countField setEnabled:YES];
    }else{
        [sortMatrix selectCellWithTag:0];
        [countField setEnabled:NO];
    }
    [window makeKeyAndOrderFront:self];
}

- (void) setSorting:sender
{
    int tag = [sortMatrix selectedTag];
    [IDLDefinition setSortMatrixItems:tag?YES:NO];
    if(tag)
        [countField setEnabled:YES];
    else
        [countField setEnabled:NO];
}

- (void)controlTextDidEndEditing:(NSNotification *)notification 
{
    int n = [countField intValue];
    [IDLDefinition setSortIfGreaterThan:n];
}

@end
