//
//  UStockWatch.m
//  IDLExplorer
//
//  Created by Victor on 1/18/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "StockWatchController.h"
#import "ConnectController.h"
#import "SymbolController.h"
#import "StockAdminController.h"
#import "QuoteItem.h"

@implementation StockWatchController

@synthesize _admin;

static NSString *kSymbolCell = @"symbol";
static int kSymbolViewTag = 101;


- (void)viewDidLoad 
{
    [super viewDidLoad];
	
	if(![self server])
		[self performSelector:@selector(connect:) withObject:self afterDelay:0.1];
//		[self connect:self];
	
}


- (void)didReceiveMemoryWarning
{
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload
{
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc
{
    // Remove ourselves from all the connections
    [self deactivate];

    [[NSNotificationCenter defaultCenter]removeObserver:self];
	[quoteArray release];
	[_server release];
	[_admin release];
	[connectController release];
	[symbolController release];
    [window release];    
    [super dealloc];
}

- (IDLInterface*) interface
{
    return [[ORB defaultORB]interfaceNamed:@"StockExample/StockWatch"];
}

- (void) notify:(NSArray*)quotes
{
    // Post notification on main thread
    [ORB postNotification:[NSNotification notificationWithName:@"QuotesDidUpdate" object:self 
													  userInfo:[NSDictionary dictionaryWithObject:quotes forKey:@"quotes"]]];
}

- server
{
	return _server;
}

- (void) setServer:server
{
	[_server release];
	_server = [server retain];
	[_server setDelegate:self];
	[iorButton setEnabled:(_server != nil)];
	[addButton setEnabled:(_server != nil)];
	[watchButton setEnabled:(_server != nil)];
	[adminButton setEnabled:(_server != nil)];
}

- (BOOL) isWatching
{
	return _watching;
}

- (void) setWatching:(BOOL)flag
{
	@try{
		if(flag){
			if(!_watching){
				[_server start:self];
				_watching = YES;
			}
		}else{
			if(_watching){
				[_server stop:self];
				_watching = NO;
			}
		}
	}@catch(NSException *e){
		UIAlertView *alert = [[UIAlertView alloc] initWithTitle:[e name] message:[e reason]
													   delegate:self cancelButtonTitle:@"OK" otherButtonTitles: nil];
		[alert show];	
		[alert release];
    }
	[watchButton setTitle:(_watching? @"Stop" : @"Watch")];
}

- (void) watch:sender
{
	[self setWatching:!_watching];
}

- (void) connect:sender
{
	if(!connectController)
		connectController = [[ConnectController alloc]initWithController:self];

	[self presentModalViewController:connectController animated:YES];
}

- (void)openStockAdmin:(id)sender
{
    if(!_admin)
        _admin = [[StockAdminController alloc]initWithController:self];
	[self presentModalViewController:_admin animated:YES];
}

- (void) addSymbolWithName:(NSString*)name
{
    @try{
		QuoteItem* quote = [_server subscribe:self symbol:[name uppercaseString]];
		if(![quoteArray containsObject:quote]){
			[self willChange:NSKeyValueChangeInsertion valuesAtIndexes:[NSIndexSet indexSetWithIndex:[quoteArray count]] forKey:@"quoteArray"];
			[quoteArray addObject:quote];
			[self didChange:NSKeyValueChangeInsertion valuesAtIndexes:[NSIndexSet indexSetWithIndex:[quoteArray count]] forKey:@"quoteArray"];
			[tableView reloadData];
		}
    }@catch(NSException *e){
		UIAlertView *alert = [[UIAlertView alloc] initWithTitle:[e name] message:[e reason]
													   delegate:self cancelButtonTitle:@"OK" otherButtonTitles: nil];
		[alert show];	
		[alert release];
    }
}

- (void) addSymbol:sender
{
	if(!symbolController)
		symbolController = [[SymbolController alloc]initWithController:self];
	[self presentModalViewController:symbolController animated:YES];
}

// CORBAObject sends this message to its delegate when invalidated
- (void) objectDidInvalidate:(NSNotification*)notification
{
	if([notification object] == _server){
		self.server = nil;
	}
}

- (void) updateQuotes:(NSNotification*)note
{
	NSArray* quotes = [[note userInfo]objectForKey:@"quotes"];
	if([quotes count] != [quoteArray count]){
		[self willChange:NSKeyValueChangeRemoval valuesAtIndexes:[NSIndexSet indexSetWithIndexesInRange:NSMakeRange(0, [quoteArray count])] forKey:@"quoteArray"];
		[quoteArray removeAllObjects];
		[self didChange:NSKeyValueChangeRemoval valuesAtIndexes:[NSIndexSet indexSetWithIndexesInRange:NSMakeRange(0, [quoteArray count])] forKey:@"quoteArray"];
		[self willChange:NSKeyValueChangeInsertion valuesAtIndexes:[NSIndexSet indexSetWithIndexesInRange:NSMakeRange(0, [quotes count])] forKey:@"quoteArray"];
		[quoteArray addObjectsFromArray:quotes];
		[self willChange:NSKeyValueChangeInsertion valuesAtIndexes:[NSIndexSet indexSetWithIndexesInRange:NSMakeRange(0, [quotes count])] forKey:@"quoteArray"];
	}else{
		[self willChange:NSKeyValueChangeReplacement valuesAtIndexes:[NSIndexSet indexSetWithIndexesInRange:NSMakeRange(0, [quotes count])] forKey:@"quoteArray"];
		[quoteArray replaceObjectsAtIndexes:[NSIndexSet indexSetWithIndexesInRange:NSMakeRange(0, [quoteArray count])] withObjects:quotes];
		[self didChange:NSKeyValueChangeReplacement valuesAtIndexes:[NSIndexSet indexSetWithIndexesInRange:NSMakeRange(0, [quotes count])] forKey:@"quoteArray"];
	}
	[tableView reloadData];
}

#pragma mark UITableDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
	return 1;
}

- (UITableViewCell *)tableView:(UITableView *)table cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	int row = [indexPath row];
	QuoteItem *item = [quoteArray objectAtIndex:row];
	UITableViewCell *cell = [table dequeueReusableCellWithIdentifier:kSymbolCell];
	if(!cell){
		cell = [[UITableViewCell alloc]initWithStyle:UITableViewCellStyleDefault reuseIdentifier:kSymbolCell];
		cell.selectionStyle = UITableViewCellSelectionStyleNone;
	}else{
		UIView *ctrl = [cell.contentView viewWithTag:kSymbolViewTag];
		[ctrl removeFromSuperview];
	}
	cell.textLabel.text = [item symbol];
	cell.textLabel.font = [UIFont boldSystemFontOfSize:17.0];
	CGRect frame = {{180.0, 12.0}, {120.0, 20.0}};
	UILabel *quote = [[UILabel alloc]initWithFrame:frame];
	quote.text = [NSString stringWithFormat:@"%.2f", [item quote]];
	quote.font = [UIFont systemFontOfSize:17.0];
	quote.tag = kSymbolViewTag;
	quote.textAlignment = UITextAlignmentRight;
	[cell.contentView addSubview:quote];
	[quote release];
	return cell;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
	return [quoteArray count];
}

#pragma mark UITableView delegate 



#pragma mark app-delegate

- (void)applicationDidReceiveMemoryWarning:(UIApplication *)application
{
	
}

- (void)applicationDidFinishLaunching:(UIApplication *)application
{
    NSString* path = [[NSBundle mainBundle]pathForResource:@"Quote" ofType:@"idl"];
    NSArray* files = [NSArray arrayWithObject:path];
    [[ORB defaultORB]loadFiles:files];
    
    // Before any connection is established
    if([[NSUserDefaults standardUserDefaults]boolForKey:@"useMainThread"])
        [ORB setHandlesRequestsOnMainThread:YES];

	quoteArray = [[NSMutableArray alloc]init];
	[[NSNotificationCenter defaultCenter] addObserver:self 
											 selector:@selector(updateQuotes:)
												 name:@"QuotesDidUpdate"
											   object:self];
	
	[window addSubview: self.view];
	[window makeKeyAndVisible];
}

@end
