/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import <ADORB/ADORB.h>
#import "NamingTest.h"
#import "NamingContextProto.h"


@implementation NamingTest

static NSString* interfaceName = @"ORBTest/NumbersIF";

- (IDLInterface*) interface
{
    return [[ORB defaultORB]interfaceNamed:interfaceName];
}

- init
{
    // Nothing to init, call 'super'
    self = [super init];
    return self;
}

- (void) run
{
	@try{
		NSString* ior;
		NSString* path = [[NSFileManager defaultManager]currentDirectoryPath];
		id nameService = [ORB resolveInitialReferences:@"NameService"];
		
		// Bind self in Naming Service, if provided
		// We don't need to activate 'self' explicitly since it will be auto-activated
		if(nameService){
			CORBAObject* ctxt = nil;
			@try{
				// Create new context in the Naming Service with the name 'test'
				ctxt = [nameService bindNewContext:[IDLNameComponent nameComponentsForPath:@"test"]];
			}@catch(NSException *localException){
				BOOL ignore = NO;
				if([[localException name]isEqualToString:CORBAUserException]){
					NSString* exceptionName = [[localException userInfo]valueForKey:@"name"];
					if([exceptionName isEqualToString:@"AlreadyBound"])
						ignore = YES;
				}
				if(!ignore){
					NSLog(@"Unable to bind new context, %@, %@, userInfo %@",
						[localException name], [localException reason], [localException userInfo]);
					[localException raise];
				}
			}
			// Bind self in Naming Context. The reference wiil be bound in created context
			[nameService rebind:[IDLNameComponent nameComponentsForPath:@"test/numbers"] object:self];
		}else
			// If no 'nameService' we have to activate 'self' explicitly to make it available
			[self activate];
		// Save IOR to the file
		ior = [[self objectReference]toString];
		if([ior writeToFile:[path stringByAppendingPathComponent:@"ORBNamingTest.ref"] atomically:NO encoding:NSUTF8StringEncoding error:NULL]){
			// Just test it. This should return our object reference, a unique CORBAObject for IOR
			id obj = [CORBAObject objectWithIOR:ior]; 
			if(obj != [self objectReference])
				NSLog(@"IT IS NOT WORKING as promised !!!");
			NSLog(@"%@: Saved IOR in './ORBNamingTest.ref'\n===> ORB is running ...\n", [self className]);
			// Run ORB
			[ORB run];
		}else
			NSLog(@"Cannot write IOR to file './ORBTest.ref'\n");
	}@catch(NSException *localException){
		NSLog(@"%@, %@", [localException name], [localException reason]);
	}
}

- (NSArray*) _getNumbers
{
    int n = 10;
    NSMutableArray* array = [NSMutableArray array];
    while(n-- > 0){
        [array addObject:[NSNumber numberWithShort:n+1]];
    }
    NSLog(@"Returning %@\n", [array componentsJoinedByString:@", "]);
    return array;
}

- (NSArray*) diag:(NSArray*)sizes
{
    NSMutableArray* diags = [NSMutableArray array];
    NSEnumerator* en = [sizes objectEnumerator];
    id size;
    while(size = [en nextObject]){
        double width = [[size valueForKey:@"width"]doubleValue];
        double height = [[size valueForKey:@"height"]doubleValue];
        double diag = sqrt(width*width + height*height);
        [diags addObject:[NSNumber numberWithDouble:diag]];
        NSLog(@"server: diag: width = %f, height = %f, diag = %f\n", width, height, diag);
    }
    return diags;
}

- (NSArray*) getDiag:(NSArray*)sizes
{
    NSMutableArray* diags = [NSMutableArray array];
    NSEnumerator* en = [sizes objectEnumerator];
    id size;
    while(size = [en nextObject]){
        double width = [[size valueForKey:@"width"]doubleValue];
        double height = [[size valueForKey:@"height"]doubleValue];
        double diag = sqrt(width*width + height*height);
        [diags addObject: [NSDictionary dictionaryWithObjectsAndKeys:
            size, @"size",
            [NSNumber numberWithDouble:diag], @"diag", nil]];
        NSLog(@"server: diag: width = %f, height = %f, diag = %f\n", width, height, diag);
    }
    return diags;
}

- copyAny:value
{
    NSLog(@"copy_any: returning\n%@",[value description]);
    return value;
}

@end
