/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "ORBTest.h"
#import <ADORB/ADORB.h>
#import "NamingContextProto.h"
#import "TestProto.h"

#define MAP_OCTET_TO_NSDATA 0

@implementation ORBTest

static NSString* interfaceName = @"ORBTest/Test";

// Provide this method so that an instance can be activated
- (IDLInterface*) interface
{
    return [[ORB defaultORB]interfaceNamed:interfaceName];
}

- init
{
    self = [super init];
	[[NSNotificationCenter defaultCenter]addObserver:self
		selector:@selector(localAddressDidChange:)
		name:@"ORBLocalAddressDidChange"
		object:nil];
	return self;
}

- (void) run
{
#if MAP_OCTET_TO_NSDATA
	[ORB setMapsOctetToNSData:YES];
#endif
	[CORBAObject setDefaultConnectTimeout:10.0];
    NSString* path = [[NSFileManager defaultManager]currentDirectoryPath];
    CORBAObject* nameService = [ORB resolveInitialReferences:@"NameService"];
    if(nameService)
        // Rebind self in the Naming Service. 'self' will be implicitly activated
        [nameService rebind:[IDLNameComponent nameComponentsForPath:@"ORBTest"] object:self];
    else
        // Otherwise we have to activate 'self' explicitly
        [self activate];

    NSString* ior = [[self objectReference]toString];
	NSAssert([CORBAObject objectWithIOR:ior] == [self objectReference], @"Failed - CORBAObject is not unique");

    // Save IOR to the file
    if([[[self objectReference]toString] writeToFile:[path stringByAppendingPathComponent:@"ORBTest.ref"]
										  atomically:NO encoding:NSUTF8StringEncoding error:NULL]){
        NSLog(@"%@: Saved IOR in './ORBTest.ref'\n===> ORB is running ...\n", [self className]);
        // Run ORB
        [ORB run];
    }else
        NSLog(@"Cannot write IOR to file './ORBTest.ref'\n");
    
}

- (void) localAddressDidChange:(NSNotification*)note
{
	@try{
		CORBAObject* nameService = [ORB resolveInitialReferences:@"NameService"];
		NSString* address = [[note userInfo]objectForKey:@"address"];
		if(address && nameService)
			[nameService rebind:[IDLNameComponent nameComponentsForPath:@"ORBTest"] object:self];
	}@catch(NSException *localException){
		NSLog(@"%@: %@", [localException name], [localException reason]);
	}
}

#pragma mark --- Implementation methods ---

- (short) addShort:(in short)a b:(in short)b sum:(out short*)sum
{
    return *sum = a + b;
}

- (long) addLong:(const long)a b:(const long)b sum:(long*)sum
{
    return *sum = a + b;
}

- (long long) addLongLong:(long long)a b:(long long)b sum:(long long *)sum
{
    return *sum = a + b;
}

- (double) addDouble:(double)a b:(double)b sum:(out double*)sum
{
    return *sum = a + b;
}

- (float) addFloat:(float)a b:(float)b sum:(float*)sum
{
    return *sum = a + b;
}

- (void) addToShort:(short*)a b:(short)b
{
    *a += b;
}

// uses selector from #pragma
- (void) addToLong:(long*)a value:(long)b
{
    *a += b;
}

- (void) addToDouble:(double*)a b:(double)b
{
    *a += b;
}

- (void) addToFloat:(float*)a b:(float)b
{
    *a += b;
}

- (void) addToLongLong:(long long*)a b:(long long)b
{
    *a += b;
}

- (double) sqrt:(double)x sqrt:(double*)b
{
    if(x > 0){
        return *b = sqrt(x);
    }else{
        [NSException raise:CORBAInvalidArgumentException
            format:@"Negative argument for 'sqrt'"];
        return 0.0;
    }
}

- (BOOL) greaterThan:(double)a b:(double)b
{
    return a > b;
}

- (const NSData*) toHex:(NSString*)str
{
    if(str)
		return [str dataUsingEncoding:NSUTF8StringEncoding];
    else
        return nil;
}

- (const NSString*) fromHex:(NSData*)data
{
	if(data)
		return [[[NSString alloc]initWithData:data encoding:NSUTF8StringEncoding]autorelease];
	else
		return nil;
}

- (const NSData*)charToOctet:(char)x
{
    char b = x;
    return [NSData dataWithBytes:&b length:1];
}

- (const NSData*) stringToUTF8:(NSString*)str
{
    return [str dataUsingEncoding:NSUTF8StringEncoding];
}

- (NSData*) stringToUnicode:(NSString*)str
{
    return [str dataUsingEncoding:NSUnicodeStringEncoding];
}

- (NSString*) UTF8ToString:(NSData*)bytes
{
    return [[[NSString alloc]initWithData:bytes encoding:NSUTF8StringEncoding]autorelease];
}

- (NSString*) unicodeToString:(NSData*)bytes
{
    return [[[NSString alloc]initWithData:bytes encoding:NSUnicodeStringEncoding]autorelease];
}

- (void) setValue:(long)value
{
    NSLog(@"Get set_value(%d)\n", value);
}

- (NSString*) whatIs:(NSString*)question context:context
{
    if([context count] > 0)
        return [NSString stringWithFormat:@"Context: %@", [context description]];
    else
        return @"I don't know";
}

- (long long) sum:(NSArray*)array
{
    NSEnumerator* en = [array objectEnumerator];
    NSNumber* obj;
    long long sum = 0;
    while(obj = [en nextObject])
        sum += [obj longValue];
    return sum;
}

- (NSString*) echo:(NSString*)string
{
	return string;
}

- (NSArray*) getAllTests
{
	return [NSArray arrayWithObject:self];
}

- (short) echoShort:(short)x result:(short*)r
{
	return *r = x;
}
- (long) echoLong:(long)x result:(long*)r
{
	return *r = x;
}
- (float) echoFloat:(float)x result:(float*)r
{
	return *r = x;
}
- (double) echoDouble:(double)x result:(double*)r
{
	return *r = x;
}
// uses selector from #pragma
- (char) echoChar:(char)x returnValue:(char*)r
{
	return *r = x;
}
- (unsigned char) echoOctet:(unsigned char)x returnValue:(unsigned char*)r
{
	return *r = x;
}
- (NSString*) echoString:(NSString*)x result:(NSString**)r
{
	return *r = x;
}

- (NSData*) echoSeqOctet:(NSData*)x result:(NSData**)r
{
	return *r = x;
}

- (void) dupChar:(char*)x
{
	*x = *x + 32;
}
- (void) dupShort:(short*)x
{
	*x = *x*2;
}

- (void) dupLong:(long * const)x
{
	*x = *x * 2;
}

- (void) dupFloat:(float*)x
{
	*x = *x * 2;
}
- (void) dupDouble:(double*)x
{
	*x = *x * 2;
}
- (void) dupString:(NSString**)x
{
	* x = [NSString stringWithFormat:@"%@-%@", *x, *x];
}
- (void) dupOctet:(unsigned char*)b
{
	*b = (*b>>4) & 0x0f | (*b << 4) & 0xf0;
}

- (void) dupSeqOctet:(NSData**)x
{
	NSMutableData* data = [NSMutableData dataWithData:*x];
	[data appendData:*x];
	*x = data;
}

- echoUnion:obj result:(id*)result
{
	return *result = obj;
}

- echoVUnion:obj result:(id*)result
{
	return *result = obj;
}

- echoUnionSequence:array result:(id*)result
{
	return *result = array;
}

- echoEnum:obj result:(id*)result
{
	return *result = obj;
}

@end
