/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import <Foundation/Foundation.h>

@class IDLInterface, CORBAObject, IDLFunction, VXMessage;

enum _CORBAHandlerMinorCode {
    CodeBase =10100,
    // INITIALIZE
    GIOPDefinitionError, // Cannot find IDL definition for a GIOP header
    // NO_IMPLEMENT
    ProfileIdNotSupported,
    // INV_OBJREF
    ObjectNotFound,
    ProfileError, // GIOP 1.2, ReferenceAddr, IOR does not match local object
    // NO_IMPLEMENT
    TargetDoesNotRespond, // Target implementation does not implement the operation
    // BAD_OPERATION
    OperationNotFound, // Target IDL interface does not have the operation
    // MARSHAL
    MarshalException,
     // IMPL_LIMIT
     AsynchCancelRequest, // Asynchronous Cancel Request not implemented
     UnknownMsgType,
     MessageError  
};


@interface VXMessageHandler : NSObject
{
@private
    id _connection;
    VXMessage* _message;
    id _requestHeader;
    id _target;
    id _targetObject;
    IDLInterface* _targetInterface;
    NSException* _exception;
	NSMutableDictionary *_requestContext;
	NSMutableDictionary *_responseContext;
	NSMutableArray* _flowStack;
	SEL _selector;
	IDLFunction* _function;
	NSMutableDictionary* _args;
    int _msgType;
    unsigned short _completionStatus;
	unsigned short _replyStatus;
}

+ handlerWithMessage:(VXMessage*)msg connection:(VXConnection*)conn;
- initWithMessage:(VXMessage*)msg connection:(VXConnection*)conn;

- (void) handleMessage;
- (BOOL) responseExpected;
- (long unsigned) messageType;
- connection;

- (long) requestId;
- (NSNumber*) request_id;
- (void) decodeRequestHeader;
- locateObjectWithObjectID:(NSData*)objectKey;

@end
